'use strict';

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _reactDom = require('react-dom');

var _objectAssign = require('object-assign');

var _objectAssign2 = _interopRequireDefault(_objectAssign);

var _dragHelper = require('drag-helper');

var _dragHelper2 = _interopRequireDefault(_dragHelper);

var _region = require('region');

var _region2 = _interopRequireDefault(_region);

var _reactStyleNormalizer = require('react-style-normalizer');

var _reactStyleNormalizer2 = _interopRequireDefault(_reactStyleNormalizer);

var _reactEventNames = require('react-event-names');

var _reactEventNames2 = _interopRequireDefault(_reactEventNames);

var _absoluteCenter = require('./absoluteCenter');

var _absoluteCenter2 = _interopRequireDefault(_absoluteCenter);

var _clamp = require('./clamp');

var _clamp2 = _interopRequireDefault(_clamp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var stringOrNumber = _react.PropTypes.oneOfType([_react.PropTypes.string, _react.PropTypes.number]);

function emptyFn() {}

function getValue(props, state) {
    var value = props.value == null ? state.value == null ? props.startValue : state.value : props.value;

    if (state.dragging) {
        value = state.value;
    }

    return value;
}

function _toValue(value, props) {
    return props.toStep((0, _clamp2.default)(value, props.startValue, props.endValue), props.step);
}

function toRange(props) {
    return Math.abs(props.endValue - props.startValue);
}

function getPercentageForValue(value, props) {
    value = value || 0;

    var range = toRange(props);
    var diff = value - props.startValue;

    return diff * 100 / range;
}

function getValueForPercentage(percentage, props, config) {
    var range = toRange(props);
    var value = percentage * range / 100;

    var noStartValue = config && config.noStartValue;
    var v = noStartValue ? value : props.startValue + value;

    return v;
}

function valueToStep(value, step) {
    if (!step) {
        return value;
    }

    var fn = function fn(value) {
        return value;
    };
    var diff;

    value = fn(value);
    diff = fn(value % step);

    if (diff > fn(step / 2)) {
        value += -diff + step;
    } else {
        value += -diff;
    }

    return value;
}

function getOffset(value, props) {
    value = getPercentageForValue(value || 0, props);

    return 100 - value;
}

function positionStyle(value, props, style) {
    style = style || {};

    var offset = getOffset(value, props);
    var horiz = props.orientation == 'horizontal';

    style[horiz ? 'width' : 'height'] = 100 - offset + '%';

    return style;
}

var DISPLAY_NAME = 'ReactSlider';

exports.default = _react2.default.createClass({

    displayName: DISPLAY_NAME,

    propTypes: {
        minValue: stringOrNumber,
        maxValue: stringOrNumber,

        startValue: stringOrNumber.isRequired,
        endValue: stringOrNumber.isRequired,

        value: stringOrNumber,

        step: stringOrNumber,
        tickStep: stringOrNumber,
        ticks: _react2.default.PropTypes.array,

        toStep: _react2.default.PropTypes.func,
        onDrag: _react2.default.PropTypes.func,
        onChange: _react2.default.PropTypes.func,

        handleSize: stringOrNumber,

        orientation: function orientation(props, propName) {
            var value = props[propName];

            if (value != 'horizontal' && value != 'vertical') {
                return new Error('orientation must be either "horizontal" or "vertical"');
            }
        }
    },

    getDefaultProps: function getDefaultProps() {
        return {
            'data-display-name': DISPLAY_NAME,

            smallTickPercentage: 80,
            orientation: 'horizontal',

            startValue: 0,
            endValue: 100,

            step: 1,

            trackRadius: 0,
            statefulDrag: true,

            enableTrackClick: true,

            defaultStyle: {},

            defaultHorizontalStyle: {
                height: 20,
                width: 200
            },

            defaultVerticalStyle: {
                height: 200,
                width: 20
            },

            defaultTrackStyle: (0, _absoluteCenter2.default)({
                position: 'relative',
                cursor: 'pointer',
                background: 'rgb(218, 218, 218)'
            }),

            defaultHorizontalTrackStyle: {
                height: 10
            },

            defaultVerticalTrackStyle: {
                width: 10
            },

            defaultTrackFillStyle: {
                position: 'absolute',
                width: '100%',
                height: '100%',
                boxSizing: 'content-box',
                background: 'rgb(120, 120, 120)'
            },

            defaultTrackLineStyle: {
                position: 'absolute'
            },
            defaultHorizontalTrackLineStyle: {
                height: '100%'
            },
            defaultVerticalTrackLineStyle: {
                width: '100%'
            },

            defaultHandleStyle: {
                position: 'absolute',
                background: 'rgb(103, 175, 233)',
                cursor: 'pointer',
                zIndex: 1
            },

            defaultOverHandleStyle: {
                background: 'rgb(118, 181, 231)'
            },

            defaultActiveHandleStyle: {
                background: 'rgb(90, 152, 202)'
            },

            defaultHorizontalHandleSize: {
                width: 10,
                height: 20
            },
            defaultVerticalHandleSize: {
                width: 20,
                height: 10
            },
            defaultHorizontalHandleStyle: {
                transform: 'translate3d(0px, -50%, 0px)',
                top: '50%'
            },
            defaultVerticalHandleStyle: {
                transform: 'translate3d(-50%, 0px, 0px)',
                left: '50%'
            },
            defaultTickStyle: {
                boxSizing: 'border-box',
                position: 'absolute',
                borderStyle: 'solid',
                borderWidth: 0
            },
            defaultHorizontalTickStyle: {
                height: '100%'
            },
            defaultVerticalTickStyle: {
                width: '100%'
            },
            defaultWrapStyle: {
                position: 'relative',
                height: '100%'
            },
            defaultTickWrapStyle: {
                position: 'absolute'
            },
            tickColor: 'rgb(172, 172, 172)',
            tickWidth: 1,
            toStep: valueToStep
        };
    },

    getInitialState: function getInitialState() {
        return {
            value: this.props.defaultValue
        };
    },

    componentWillReceiveProps: function componentWillReceiveProps(newProps) {
        this.tickCache = null;
    },

    render: function render() {

        var props = this.prepareProps(this.props, this.state);
        var track = this.renderTrack(props, this.state);

        var ticks = this.tickCache ? this.tickCache : this.renderTicks(props, this.state);

        this.tickCache = ticks;

        var wrapStyle = (0, _objectAssign2.default)({}, props.defaultWrapStyle, props.wrapStyle);

        return _react2.default.createElement(
            'div',
            props,
            _react2.default.createElement(
                'div',
                { className: 'z-wrap', style: wrapStyle },
                ticks,
                track
            )
        );
    },

    prepareProps: function prepareProps(thisProps, state) {
        var props = {};

        (0, _objectAssign2.default)(props, thisProps);

        props.horizontal = props.orientation === 'horizontal';

        props.style = this.prepareStyle(props);
        props.handleStyle = this.prepareHandleStyle(props);
        props.trackStyle = this.prepareTrackStyle(props);
        props.trackLineStyle = this.prepareTrackLineStyle(props);
        props.trackFillStyle = this.prepareTrackFillStyle(props);
        props.tickStyle = this.prepareTickStyle(props);

        props.value = this.toValue(getValue(props, state), props);

        props.className = this.prepareClassName(props);

        props.onDrag = null;

        return props;
    },

    prepareClassName: function prepareClassName(props) {
        var className = props.className || '';
        var horiz = props.horizontal;
        var orientationClass = horiz ? 'z-orientation-horizontal' : 'z-orientation-vertical';

        className += ' ' + orientationClass;
        className += ' z-slider';

        return className;
    },

    toValue: function toValue(value, props) {
        props = props || this.props;

        return _toValue(value, props);
    },

    prepareStyle: function prepareStyle(props) {
        var orientationStyle = props.horizontal ? props.defaultHorizontalStyle : props.defaultVerticalStyle;

        return (0, _reactStyleNormalizer2.default)((0, _objectAssign2.default)({}, props.defaultStyle, orientationStyle, props.style));
    },

    prepareHandleStyle: function prepareHandleStyle(props) {
        var horiz = props.horizontal;
        var orientationStyle = horiz ? props.defaultHorizontalHandleStyle : props.defaultVerticalHandleStyle;

        var defaultActiveHandleStyle;
        var activeHandleStyle;

        if (this.state.handleMouseDown) {
            defaultActiveHandleStyle = props.defaultActiveHandleStyle;
            activeHandleStyle = props.activeHandleStyle;
        }

        var defaultOverHandleStyle;
        var overHandleStyle;

        if (this.state.handleMouseOver) {
            defaultOverHandleStyle = props.defaultOverHandleStyle;
            overHandleStyle = props.overHandleStyle;
        }

        var handleStyle = (0, _objectAssign2.default)({}, props.defaultHandleStyle, orientationStyle, defaultOverHandleStyle, defaultActiveHandleStyle, props.handleStyle, overHandleStyle, activeHandleStyle);
        var handleSize = this.getHandleSize(props);

        handleStyle.width = typeof handleStyle.width === 'undefined' ? handleSize.width : handleStyle.width;
        handleStyle.height = typeof handleStyle.height === 'undefined' ? handleSize.height : handleStyle.height;

        return (0, _reactStyleNormalizer2.default)(handleStyle);
    },

    prepareTickStyle: function prepareTickStyle(props) {
        var horiz = props.horizontal;
        var side = horiz ? 'left' : 'top';
        var orientationStyle = horiz ? props.defaultHorizontalTickStyle : props.defaultVerticalTickStyle;
        var style = (0, _objectAssign2.default)({}, props.defaultTickStyle, orientationStyle, props.tickStyle);

        if (horiz) {
            style.borderLeftWidth = style.borderLeftWidth || props.tickWidth;
            style.borderLeftColor = style.borderLeftColor || props.tickColor;
            style.marginLeft = -props.tickWidth / 2;
        } else {
            style.borderTopWidth = style.borderTopWidth || props.tickWidth;
            style.borderTopColor = style.borderTopColor || props.tickColor;
            style.marginTop = -props.tickWidth / 2;
        }

        return (0, _reactStyleNormalizer2.default)(style);
    },

    prepareTrackStyle: function prepareTrackStyle(props) {
        var horiz = props.horizontal;
        var trackOrientationStyle = horiz ? props.defaultHorizontalTrackStyle : props.defaultVerticalTrackStyle;
        var trackStyle = (0, _objectAssign2.default)({}, props.defaultTrackStyle, trackOrientationStyle, props.trackStyle);

        if (props.trackRadius) {
            trackStyle.borderRadius = typeof trackStyle.borderRadius === 'undefined' ? props.trackRadius : trackStyle.borderRadius;
        }

        return (0, _reactStyleNormalizer2.default)(trackStyle);
    },

    prepareTrackLineStyle: function prepareTrackLineStyle(props) {
        var horiz = props.horizontal;

        var orientationStyle = horiz ? props.defaultHorizontalTrackLineStyle : props.defaultVerticalTrackLineStyle;

        var trackLineStyle = (0, _objectAssign2.default)({}, props.defaultTrackLineStyle, orientationStyle, props.trackLineStyle);

        return (0, _reactStyleNormalizer2.default)(trackLineStyle);
    },

    prepareTrackFillStyle: function prepareTrackFillStyle(props) {
        var horiz = props.horizontal;
        var trackFillStyle = (0, _objectAssign2.default)({}, props.defaultTrackFillStyle, props.trackFillStyle);

        if (props.trackRadius) {
            if (horiz) {
                trackFillStyle.borderTopLeftRadius = trackFillStyle.borderTopLeftRadius || props.trackRadius;
                trackFillStyle.borderBottomLeftRadius = trackFillStyle.borderBottomLeftRadius || props.trackRadius;
            } else {
                trackFillStyle.borderTopLeftRadius = trackFillStyle.borderTopLeftRadius || props.trackRadius;
                trackFillStyle.borderTopRightRadius = trackFillStyle.borderTopRightRadius || props.trackRadius;
            }
        }

        return trackFillStyle;
    },

    renderTrack: function renderTrack(props, state) {

        var value = props.value;
        var horiz = props.horizontal;

        var trackStyle = props.trackStyle;
        var trackLineStyle = props.trackLineStyle;
        var trackFillStyle = props.trackFillStyle;

        positionStyle(value, props, trackFillStyle);

        var handleSize = {
            width: props.handleStyle.width,
            height: props.handleStyle.height
        };
        var size;

        if (horiz) {
            size = handleSize.width / 2;

            trackLineStyle.left = size;
            trackLineStyle.right = size;
            trackFillStyle.paddingLeft = size;
            trackFillStyle.marginLeft = -size;
        } else {

            size = handleSize.height / 2;

            trackLineStyle.top = size;
            trackLineStyle.bottom = size;
            trackFillStyle.paddingTop = size;
            trackFillStyle.marginTop = -size;
        }

        var handle = this.renderHandle(props, state);

        return _react2.default.createElement(
            'div',
            { className: 'z-track', style: trackStyle, onMouseDown: this.handleTrackMouseDown.bind(this, props) },
            _react2.default.createElement(
                'div',
                { ref: 'trackLine', className: 'z-track-line', style: trackLineStyle },
                _react2.default.createElement('div', { className: 'z-track-fill', style: trackFillStyle }),
                handle
            )
        );
    },

    renderTicks: function renderTicks(props) {

        var horiz = props.orientation === 'horizontal';
        var ticks = props.ticks = this.prepareTicks(props);
        var handleSize = {
            width: props.handleStyle.width,
            height: props.handleStyle.height
        };

        if (!ticks || !ticks.length) {
            return;
        }

        var tickWrapStyle = (0, _objectAssign2.default)({}, props.defaultTickWrapStyle, props.tickWrapStyle);

        if (horiz) {
            tickWrapStyle.left = tickWrapStyle.right = handleSize.width / 2;
            tickWrapStyle.height = '100%';
        } else {
            tickWrapStyle.top = tickWrapStyle.bottom = handleSize.height / 2;
            tickWrapStyle.width = '100%';
        }

        return _react2.default.createElement(
            'div',
            { style: tickWrapStyle },
            ticks.map(this.renderTick.bind(this, props))
        );
    },

    renderTick: function renderTick(props, tick) {

        var tickValue;
        var tickStyle;
        var type = 'big';

        if ((typeof tick === 'undefined' ? 'undefined' : _typeof(tick)) == 'object') {
            tickValue = tick.value;
            tickStyle = tick.style;
            type = tick.type || type;
        } else {
            tickValue = tick;
        }

        if (type == 'small') {
            tickStyle = tickStyle || {};

            var percentage = parseFloat(props.smallTickPercentage);
            tickStyle[props.horizontal ? 'top' : 'left'] = (100 - percentage) / 2 + '%';
            tickStyle[props.horizontal ? 'height' : 'width'] = percentage + '%';
        }

        var side = props.horizontal ? 'right' : 'bottom';
        var style = (0, _objectAssign2.default)({}, props.tickStyle, tickStyle);

        style[side] = getOffset(tickValue, props) + '%';

        var tickProps = {
            key: tickValue + '-tick',
            value: tickValue,
            orientation: props.orientation,
            type: type,
            'data-value': tickValue,
            style: style
        };

        var defaultFactory = _react2.default.DOM.div;
        var factory = props.tickFactory || defaultFactory;

        var result = factory(tickProps);

        if (result === undefined) {
            result = defaultFactory(tickProps);
        }

        return result;
    },

    prepareTicks: function prepareTicks(props) {
        var ticks = props.ticks;

        if (props.tickStep) {

            var ticks = [];
            var start = props.startValue;
            var end = props.endValue;
            var step = props.tickStep * 1;

            while (start + step < end) {
                start += step;
                ticks.push(start);
            }
        }

        return ticks;
    },

    getHandleSize: function getHandleSize(props) {
        var obj = props.orientation === 'horizontal' ? props.defaultHorizontalHandleSize : props.defaultVerticalHandleSize;
        return {
            width: props.handleWidth || props.handleSize || obj.width,
            height: props.handleHeight || props.handleSize || obj.height
        };
    },

    renderHandle: function renderHandle(props, state) {
        var value = props.value;

        var handleStyle = props.handleStyle;
        var handleSize = this.getHandleSize(props);

        handleSize.width = handleStyle.width;
        handleSize.height = handleStyle.height;

        var offset = 100 - getOffset(value, props) + '%';

        if (props.horizontal) {
            handleStyle.left = offset;
            handleStyle.marginLeft = -handleSize.width / 2;
            handleStyle.marginTop = handleStyle.marginBottom = 'auto';
            handleStyle.transform = 'translate3d(0px, -50%, 0px)';
        } else {
            handleStyle.top = offset;
            handleStyle.marginTop = -handleSize.height / 2;
            handleStyle.marginLeft = handleStyle.marginRight = 'auto';
            handleStyle.transform = 'translate3d(-50%, 0px, 0px)';
        }

        if (props.trackRadius) {
            handleStyle.borderRadius = typeof handleStyle.borderRadius === 'undefined' ? props.trackRadius : handleStyle.borderRadius;
        }

        var handleProps = {
            ref: 'handle',
            className: 'z-handle',
            'data-value': value,
            orientation: props.orientation,
            value: value,
            dragging: state.dragging,
            mouseDown: state.handleMouseDown,
            sliderProps: props,
            style: (0, _reactStyleNormalizer2.default)(handleStyle),
            onMouseEnter: this.handleMouseEnter,
            onMouseLeave: this.handleMouseLeave
        };

        handleProps[_reactEventNames2.default.onMouseDown] = this.handleMouseDown.bind(this, props);

        var defaultFactory = _react2.default.DOM.div;
        var factory = props.handleFactory || defaultFactory;

        var result = factory(handleProps);

        if (result === undefined) {
            result = defaultFactory(handleProps);
        }

        return result;
    },

    handleMouseEnter: function handleMouseEnter() {
        this.setState({
            handleMouseOver: true
        });
    },

    handleMouseLeave: function handleMouseLeave() {
        this.setState({
            handleMouseOver: false
        });
    },

    handleTrackMouseDown: function handleTrackMouseDown(props, event) {
        if (!props.enableTrackClick) {
            return;
        }
        var horiz = props.orientation === 'horizontal';
        var region = _region2.default.from(this.getTrackDOMNOde());
        var dragSize = this.getAvailableDragSize(props);
        var handleSize = props.handleStyle[horiz ? 'width' : 'height'];

        var offset = horiz ? event.pageX - region.left : event.pageY - region.top;

        var percentage = offset * 100 / dragSize;

        this.setValue(getValueForPercentage(percentage, props));
    },

    handleMouseDown: function handleMouseDown(props, event) {
        event.preventDefault();
        event.stopPropagation();

        this.setupDrag(event, props);
    },

    getHandle: function getHandle() {
        return (0, _reactDom.findDOMNode)(this.refs.handle);
    },

    getInitialDragValue: function getInitialDragValue(props, state) {
        var initialValue = typeof this.state.value != 'undefined' ? this.state.value : typeof props.defaultValue == 'undefined' ? props.value : props.defaultValue;

        return initialValue;
    },

    getTrackDOMNOde: function getTrackDOMNOde() {
        return (0, _reactDom.findDOMNode)(this.refs.trackLine);
    },

    getAvailableDragSize: function getAvailableDragSize(props) {
        var horiz = props.orientation === 'horizontal';
        var region = _region2.default.from(this.getTrackDOMNOde());

        return horiz ? region.width : region.height;
    },

    getRegion: function getRegion() {
        return _region2.default.from((0, _reactDom.findDOMNode)(this));
    },

    setupDrag: function setupDrag(event, props, initialDiff) {

        initialDiff = initialDiff || 0;

        var horiz = props.orientation === 'horizontal';

        var targetRegion = _region2.default.from(this.getHandle());
        var constrainRegion = _region2.default.from((0, _reactDom.findDOMNode)(this));

        var dragSize = this.getAvailableDragSize(props);
        var initialValue = props.value;

        this.setState({
            handleMouseDown: true
        });

        (0, _dragHelper2.default)(event, {

            scope: this,
            region: targetRegion,
            constrainTo: constrainRegion,

            onDragStart: function onDragStart() {
                this.setState({
                    dragging: true,
                    value: props.value
                });
            },

            onDrag: function onDrag(event, config) {
                var diff = (horiz ? config.diff.left : config.diff.top) + initialDiff;

                var percentage = diff * 100 / dragSize;
                var diffValue = getValueForPercentage(percentage, props, { noStartValue: true });

                this.setValue(initialValue + diffValue, { onDrag: true });
            },

            onDrop: function onDrop() {

                var value = this.state.dragging ? this.state.value : props.value;

                var state = {
                    dragging: false,
                    handleMouseDown: false
                };

                this.setState(state);
                this.notify(value);
            }
        });
    },

    notify: function notify(value) {
        ;(this.props.onChange || emptyFn)(value);
    },

    setValue: function setValue(value, config) {

        var props = this.props;
        var newValue = this.toValue(value, props);

        var onDrag = config && config.onDrag;

        if (typeof props.defaultValue != 'undefined' || this.props.statefulDrag && onDrag) {
            this.setState({
                value: newValue
            });
        }

        if (onDrag) {
            ;(props.onDrag || emptyFn)(newValue, props);
        } else {
            this.notify(newValue);
        }
    }
});